searchData={"items":[{"type":"module","title":"Storex","doc":"","ref":"Storex.html"},{"type":"function","title":"Storex.mutate/3","doc":"Mutate store from elixir.\n\nInvoke mutation callback globally across specified store asynchronously:\n```elixir\nStorex.mutate(\"ExampleApp.Store\", \"reload\", [\"params\"])\n```","ref":"Storex.html#mutate/3"},{"type":"function","title":"Storex.mutate/4","doc":"Mutate store from elixir.\n\nInvoke mutation callback by specified key and store asynchronously:\n```elixir\nStorex.mutate(\"user_id\", \"ExampleApp.Store\", \"reload\", [\"params\"])\n```","ref":"Storex.html#mutate/4"},{"type":"module","title":"Storex.DTS","doc":"","ref":"Storex.DTS.html"},{"type":"function","title":"Storex.DTS.convert/1","doc":"","ref":"Storex.DTS.html#convert/1"},{"type":"module","title":"Storex.Diff","doc":"","ref":"Storex.Diff.html"},{"type":"function","title":"Storex.Diff.check/2","doc":"Check difference between two arguments.\n\n```elixir\nStorex.Diff.check(%{name: \"John\"}, %{name: \"Adam\"})\n[%{a: \"u\", p: [:name], t: \"Adam\"}]\n```\n\nResult explanation:\n```\na: action\n  n - none\n  u - update\n  d - delete\n  i - insert\nt: to\np: path\n```","ref":"Storex.Diff.html#check/2"},{"type":"module","title":"Storex.HTTP","doc":"","ref":"Storex.HTTP.html"},{"type":"function","title":"Storex.HTTP.init_store/2","doc":"","ref":"Storex.HTTP.html#init_store/2"},{"type":"module","title":"Storex.Plug","doc":"Add Storex to your Plug application, to handle WebSocket connections.\n\nExample for Phoenix Endpoint:\n\n```elixir\ndefmodule YourAppWeb.Endpoint do\n  use Phoenix.Endpoint, otp_app: :your_app\n\n  plug Storex.Plug\n\n  # ...\nend\n```","ref":"Storex.Plug.html"},{"type":"module","title":"Options - Storex.Plug","doc":"- `:path` - The path to mount the Storex handler. Default is `\"/storex\"`.","ref":"Storex.Plug.html#module-options"},{"type":"module","title":"Storex.Socket","doc":"Socket connection handler.\n\n  Error codes:\n  - `4000`: Store is not set.\n  - `4001`: Store is not defined or can't be compiled.","ref":"Storex.Socket.html"},{"type":"behaviour","title":"Storex.Store","doc":"","ref":"Storex.Store.html"},{"type":"callback","title":"Storex.Store.init/2","doc":"Called when store session starts.","ref":"Storex.Store.html#c:init/2"},{"type":"callback","title":"Storex.Store.mutation/5","doc":"","ref":"Storex.Store.html#c:mutation/5"},{"type":"callback","title":"Storex.Store.terminate/3","doc":"Called when store session ends.","ref":"Storex.Store.html#c:terminate/3"},{"type":"extras","title":"Storex","doc":"# Storex\n\n![Elixir CI](https://github.com/nerdslabs/storex/workflows/Elixir%20CI/badge.svg) [![Downloads](https://img.shields.io/hexpm/dt/storex.svg)](https://hex.pm/packages/storex)\n\nStorex is a frontend store with state management handled on the backend. It allows you to update the store state both from the frontend and backend, with all communication occurring over WebSocket.\n\nImportant: Storex is currently under active development. We encourage you to report any issues or submit feature requests [here](https://github.com/nerdslabs/storex/issues/new).","ref":"readme.html"},{"type":"extras","title":"Why Storex? - Storex","doc":"","ref":"readme.html#why-storex"},{"type":"extras","title":"Features - Storex","doc":"- Efficient state management: Only the differences (diffs) in the store state are sent with each mutation, minimizing data transfer.\n- Real-time updates: State changes are immediately reflected across all connected clients via WebSocket communication.\n- Backend-driven state: Storex allows both the frontend and backend to update the store state seamlessly.\n- Lightweight and fast: Designed for minimal overhead, ensuring rapid state updates and communication.","ref":"readme.html#features"},{"type":"extras","title":"Key Differences from Phoenix LiveView - Storex","doc":"Phoenix LiveView is a powerful tool for building rich, interactive web applications without writing custom JavaScript. However, as your application grows, managing complex client-side state across multiple LiveViews or components can become challenging. This is where Storex comes in.\n\n- Client-Side State Management: While Phoenix LiveView handles server-side rendering and event handling, Storex focuses on managing state on the client side. It allows you to keep your client-side state in sync with the server, but with more flexibility in how that state is stored, updated, and accessed.\n- Decoupled State Logic: Storex decouples state management from the LiveView itself, enabling you to manage state across multiple components or even across the entire application. This contrasts with LiveView, where state is typically tied to a specific LiveView process.\n- Predictable State Updates: Storex follows a predictable, unidirectional data flow similar to Redux. This makes it easier to reason about state changes and debug issues, especially in complex applications.\n- Extensibility: Storex is designed to be highly extensible, allowing you to integrate it with other tools and libraries in the Elixir ecosystem. You can also define custom middleware to handle side effects, logging, or other tasks.\n\nFor an overview of Storex in action, check out the example provided [here](https://storex-phoenix-ssr-spring-sky-1412.fly.dev/).","ref":"readme.html#key-differences-from-phoenix-liveview"},{"type":"extras","title":"Basic usage - Storex","doc":"","ref":"readme.html#basic-usage"},{"type":"extras","title":"Installation - Storex","doc":"Add **storex** to deps in `mix.exs`:\n\n```elixir\ndefp deps do\n  [{:storex, \"~> 0.5.0\"}]\nend\n```\n\nAlso you need to add **storex** to `package.json` dependencies:\n\n```javascript\n{\n  \"storex\": \"file:../deps/storex\",\n}\n```","ref":"readme.html#installation"},{"type":"extras","title":"Add storex websocket handler - Storex","doc":"You need to add handler `Storex.Handler.Plug` or `Storex.Handler.Cowboy`.\n\n**Phoenix:**\n```elixir\ndefmodule YourAppWeb.Endpoint do\n  use Phoenix.Endpoint, otp_app: :your_app\n\n  plug Storex.Plug, path: \"/storex\"\n\n  # ...\nend\n```\n\n**Cowboy:**\n```elixir\n:cowboy_router.compile([\n  {:_, [\n    # ...\n    {\"/storex\", Storex.Handler.Cowboy, []},\n    # ...\n  ]}\n])\n```\n\n> [!IMPORTANT]\n> Cowboy doesn't support the Node.js (HTTP Only) connector","ref":"readme.html#add-storex-websocket-handler"},{"type":"extras","title":"Create store - Storex","doc":"To create a store you need to create new elixir module with `init/2` which is called when a page is loaded, every time websocket is connected it generates session_id and passes it as the first argument, params are from Javascript store declaration. `init/2` callback need to return one of this tuples:\n\n- `{:ok, state}` - for initial state\n- `{:ok, state, key}` - for initial state with `key` which can be used as selector for future mutations\n- `{:error, reason}` - to send error message to frontend on initialization\n\nNext, you can declare `mutation/5` where the first argument is mutation name, second is data passed to mutation, next two params are same like in `init/2`, the last one is the current state of the store.\n\n```elixir\ndefmodule ExampleApp.Store.Counter do\n  use Storex.Store\n\n  def init(session_id, params) do\n    {:ok, 0}\n  end\n\n  # `increase` is mutation name, `data` is payload from front-end, `session_id` is current session id of connecton, `initial_params` with which store was initialized, `state` is store current state.\n  def mutation(\"increase\", _data, _session_id, _initial_params, state) do\n    state = state + 1\n\n    {:noreply, state}\n  end\n\n  def mutation(\"decrease\", _data, _session_id, _initial_params, state) do\n    state = state - 1\n\n    {:reply, \"message\", state}\n  end\n\n  def mutation(\"set\", [number], _session_id, _initial_params, state) do\n    {:noreply, number}\n  end\nend\n```","ref":"readme.html#create-store"},{"type":"extras","title":"Connect to store - Storex","doc":"You have to connect the newly created store with a frontend side to be able to synchronise the state: `params` are passed as second argument in store `init/2` and as third in `mutation/5`. You can subscribe to changes inside store state by passing option `subscribe` with function as a value.\n\n```typescript\nimport useStorex from 'storex'\n\nconst store = useStorex({\n  store: 'ExampleApp.Store.Counter',\n  params: {}\n})\n```","ref":"readme.html#connect-to-store"},{"type":"extras","title":"Mutate store - Storex","doc":"You can mutate store from javascript with store instance:\n\n```typescript\nstore.commit(\"increase\")\nstore.commit(\"decrease\").then((response) => {\n  response // Reply from elixir\n})\nstore.commit(\"set\", 10)\n```\n\nOr directly from elixir:\n\n```elixir\nStorex.mutate(store, \"increase\", [])\nStorex.mutate(store, \"set\", [10])\nStorex.mutate(key, store, \"increase\", [])\nStorex.mutate(key, store, \"set\", [10])\n```","ref":"readme.html#mutate-store"},{"type":"extras","title":"Subscribe to store changes - Storex","doc":"You can subscribe to store state changes in javascript with function subscribe:\n\n```typescript\nstore.subscribe((state) => {\n  const state = state\n})\n```\n\nYou can also subscribe to events after store is created:\n\n```typescript\nstore.onConnected(() => {\n  console.log('connected')\n})\n\nstore.onError((error) => {\n  console.log('error', error)\n})\n\nstore.onDisconnected((closeEvent) => {\n  console.log('disconnected', closeEvent)\n})\n```","ref":"readme.html#subscribe-to-store-changes"},{"type":"extras","title":"Connectors - Storex","doc":"The default export of `useStorex` uses WebSocket connections only, you can extend it by using custom connector.","ref":"readme.html#connectors"},{"type":"extras","title":"Websocket - Storex","doc":"```typescript\nimport { prepare, socketConnector } from 'storex';\n\nconst connector = socketConnector({ address: 'wss://myapi.com/storex' });\nconst { useStorex } = prepare({ /* global params */ }, connector);\n\nconst myStore = useStorex ({\n  store: 'myStoreName',\n  params: { /* store-specific params */ }\n});\n```","ref":"readme.html#websocket"},{"type":"extras","title":"Node.js (HTTP Only) - Storex","doc":"**Node.js connector require Node.js installed on server which running application**\n\n```typescript\nimport { prepare, httpConnector } from 'storex';\n\nconst connector = httpConnector({ address: 'http://myapi.com/storex' });\nconst { useStorex } = prepare({}, connector);\n\nconst myStore = useStorex ({\n  store: 'myStoreName',\n  params: { /* store-specific params */ }\n});\n\n// Subscribe to state changes\nmyStore.subscribe((state) => {\n  console.log('New state:', state);\n});\n\n// Handle errors\nmyStore.onError((error) => {\n  console.error('An error occurred:', error);\n});\n```\n\n> [!IMPORTANT]\n> Mutations are not supported in HTTP mode\n> myStore.commit() will not work as expected","ref":"readme.html#node-js-http-only"},{"type":"extras","title":"Configuration - Storex","doc":"","ref":"readme.html#configuration"},{"type":"extras","title":"Session id generation library - Storex","doc":"You can change library which generate session id for stores. Module needs to have **generate/0** method.\n\n```elixir\nconfig :storex, :session_id_library, Ecto.UUID\n```","ref":"readme.html#session-id-generation-library"},{"type":"extras","title":"Default params - Storex","doc":"You can set default params for all stores when preparing the Storex instance. These params will be passed to each store.\n\n```typescript\nconst { useStorex } = prepare({ jwt: 'someJWT' }, connector);\n```","ref":"readme.html#default-params"},{"type":"extras","title":"Custom store address - Storex","doc":"You can specify a custom address when creating the connector:\n\n```typescript\nconst connector = socketConnector({ address: 'wss://myapi.com/storex' });\n// OR\nconst connector = httpConnector({ address: 'http://myapi.com/storex' });\n```","ref":"readme.html#custom-store-address"}],"content_type":"text/markdown","producer":{"name":"ex_doc","version":[48,46,51,52,46,50]}}